syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface TrackVisual
/// \brief Message for a tracking a rendering::Visual with a rendering::Camera

import "vector3d.proto";

message TrackVisual
{
  /// \brief Name of the visual to track
  required string name              = 1;

  /// \brief Id of the visual to track
  optional uint32 id                = 2;

  /// \brief True to have the tracking camera inherit the orientation of
  /// the tracked visual.
  optional bool inherit_orientation = 3;

  /// \brief Minimum follow distance
  optional double min_dist          = 4;

  /// \brief Maximum follow distance
  optional double max_dist          = 5;

  /// \brief If set to true, the position of the camera is fixed.
  optional bool static              = 6;

  /// \brief If set to true, the position of the camera is relative to the
  /// model reference frame.
  optional bool use_model_frame     = 7;

  /// \brief Position of the camera.
  optional Vector3d xyz             = 8;

  /// \brief If set to true, the camera inherits the yaw rotation of the model.
  optional bool inherit_yaw         = 9;
}
