export GAZEBO_MASTER_URI=http://localhost:11345
export GAZEBO_MODEL_DATABASE_URI=http://models.gazebosim.org
export GAZEBO_RESOURCE_PATH=D:\bld\gazebo_1600638113770\_h_env\Library\share\gazebo-11:${GAZEBO_RESOURCE_PATH}
export GAZEBO_PLUGIN_PATH=D:\bld\gazebo_1600638113770\_h_env\Library\bin\gazebo-11\plugins:${GAZEBO_PLUGIN_PATH}
export GAZEBO_MODEL_PATH=D:\bld\gazebo_1600638113770\_h_env\Library\share\gazebo-11\models:${GAZEBO_MODEL_PATH}
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:D:\bld\gazebo_1600638113770\_h_env\Library\bin\gazebo-11\plugins
export OGRE_RESOURCE_PATH=D:\bld\gazebo_1600638113770\_h_env\Library\bin
