syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface LogicalCameraImage
/// \brief Information about models seen by a LogicalCameraSensor

import "pose.proto";

message LogicalCameraImage
{
  /// \brief Information about a model that is reported by a
  /// LogicalCameraSensor
  message Model
  {
    /// \brief Name of the detected model
    required string name = 1;

    /// \brief Pose of the detected model. The pose is relative to the
    /// logical camera's pose.
    required Pose pose   = 2;
  }

  /// \brief Pose of the logical camera.
  required Pose pose   = 1;

  /// \brief All the models seen by the LogicalCamera
  repeated Model model = 2;
}
