syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Sonar
/// \brief Message for a sonar value

import "pose.proto";
import "vector3d.proto";

message Sonar
{
  required string frame     = 1;
  required Pose world_pose  = 2;
  required double range_min = 3;
  required double range_max = 4;
  required double radius    = 5;
  required double range     = 6;

  /// Location of the contact relative to sensor origin.
  optional Vector3d contact  = 7;

  /// \brief The sonar collision shape.
  ///        possible values are "cone", "sphere".
  ///        If you set this value to "cone" you need to specify
  ///        the `radius`.
  required string geometry  = 8;
}
