#ifndef GAZEBO_GUI_HH_
#define GAZEBO_GUI_HH_

// Automatically generated
#include "gazebo/gui/qt.h"
#include "gazebo/gui/Conversions.hh"
#include "gazebo/gui/EntityMaker.hh"
#include "gazebo/gui/GuiIface.hh"
#include "gazebo/gui/GuiEvents.hh"
#include "gazebo/gui/GuiPlugin.hh"
#include "gazebo/gui/GuiTypes.hh"
#include "gazebo/gui/KeyEventHandler.hh"
#include "gazebo/gui/LightMaker.hh"
#include "gazebo/gui/ModelAlign.hh"
#include "gazebo/gui/ModelManipulator.hh"
#include "gazebo/gui/ModelSnap.hh"
#include "gazebo/gui/MouseEventHandler.hh"
#include "gazebo/gui/ModelMaker.hh"
#include "gazebo/gui/SpaceNav.hh"
#include "gazebo/gui/Actions.hh"
#include "gazebo/gui/AlignWidget.hh"
#include "gazebo/gui/ApplyWrenchDialog.hh"
#include "gazebo/gui/CloneWindow.hh"
#include "gazebo/gui/ConfigWidget.hh"
#include "gazebo/gui/DataLogger.hh"
#include "gazebo/gui/Editor.hh"
#include "gazebo/gui/GLWidget.hh"
#include "gazebo/gui/InsertModelWidget.hh"
#include "gazebo/gui/JointControlWidget.hh"
#include "gazebo/gui/LayersWidget.hh"
#include "gazebo/gui/LogPlayWidget.hh"
#include "gazebo/gui/MainWindow.hh"
#include "gazebo/gui/ModelListWidget.hh"
#include "gazebo/gui/ModelRightMenu.hh"
#include "gazebo/gui/RenderWidget.hh"
#include "gazebo/gui/SaveEntityDialog.hh"
#include "gazebo/gui/SplashScreen.hh"
#include "gazebo/gui/TimePanel.hh"
#include "gazebo/gui/TimeWidget.hh"
#include "gazebo/gui/ToolsWidget.hh"
#include "gazebo/gui/TopicSelector.hh"
#include "gazebo/gui/TopToolbar.hh"
#include "gazebo/gui/UserCmdHistory.hh"
#include "gazebo/gui/ViewAngleWidget.hh"


#endif
