syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface ServerControl
/// \brief A message that allows for control of the server functions


message ServerControl
{
  optional string save_world_name = 1;
  optional string save_filename   = 2;
  optional string open_filename   = 3;
  optional bool new_world         = 4;
  optional bool stop              = 5;
  optional bool clone             = 6;
  optional uint32 new_port        = 7;
}
