// Automatically generated
// avoid collision from Mac OS X's ConditionalMacros.h
// see gazebo issue #1289
#ifdef __MACH__
# undef TYPE_BOOL
#endif
#include "altimeter.pb.h"
#include "any.pb.h"
#include "atmosphere.pb.h"
#include "axis.pb.h"
#include "battery.pb.h"
#include "boxgeom.pb.h"
#include "camerasensor.pb.h"
#include "camera_cmd.pb.h"
#include "camera_lens.pb.h"
#include "cessna.pb.h"
#include "collision.pb.h"
#include "color.pb.h"
#include "contact.pb.h"
#include "contacts.pb.h"
#include "contactsensor.pb.h"
#include "cylindergeom.pb.h"
#include "density.pb.h"
#include "diagnostics.pb.h"
#include "distortion.pb.h"
#include "empty.pb.h"
#include "factory.pb.h"
#include "fluid.pb.h"
#include "fog.pb.h"
#include "friction.pb.h"
#include "geometry.pb.h"
#include "gps.pb.h"
#include "gps_sensor.pb.h"
#include "gui_camera.pb.h"
#include "gui.pb.h"
#include "gz_string.pb.h"
#include "gz_string_v.pb.h"
#include "header.pb.h"
#include "heightmapgeom.pb.h"
#include "hydra.pb.h"
#include "imagegeom.pb.h"
#include "image.pb.h"
#include "image_stamped.pb.h"
#include "images_stamped.pb.h"
#include "imu.pb.h"
#include "imu_sensor.pb.h"
#include "inertial.pb.h"
#include "int.pb.h"
#include "joint.pb.h"
#include "joint_animation.pb.h"
#include "joint_cmd.pb.h"
#include "joint_wrench.pb.h"
#include "joint_wrench_stamped.pb.h"
#include "joystick.pb.h"
#include "laserscan.pb.h"
#include "laserscan_stamped.pb.h"
#include "light.pb.h"
#include "link.pb.h"
#include "link_data.pb.h"
#include "log_control.pb.h"
#include "log_playback_control.pb.h"
#include "log_playback_stats.pb.h"
#include "log_status.pb.h"
#include "logical_camera_image.pb.h"
#include "logical_camera_sensor.pb.h"
#include "magnetometer.pb.h"
#include "material.pb.h"
#include "meshgeom.pb.h"
#include "model.pb.h"
#include "model_configuration.pb.h"
#include "model_v.pb.h"
#include "packet.pb.h"
#include "physics.pb.h"
#include "param.pb.h"
#include "param_v.pb.h"
#include "planegeom.pb.h"
#include "pid.pb.h"
#include "plugin.pb.h"
#include "pointcloud.pb.h"
#include "polylinegeom.pb.h"
#include "pose.pb.h"
#include "pose_animation.pb.h"
#include "pose_stamped.pb.h"
#include "pose_trajectory.pb.h"
#include "pose_v.pb.h"
#include "poses_stamped.pb.h"
#include "projector.pb.h"
#include "propagation_particle.pb.h"
#include "propagation_grid.pb.h"
#include "publishers.pb.h"
#include "publish.pb.h"
#include "quaternion.pb.h"
#include "sonar.pb.h"
#include "sonar_stamped.pb.h"
#include "raysensor.pb.h"
#include "request.pb.h"
#include "response.pb.h"
#include "rest_response.pb.h"
#include "rest_login.pb.h"
#include "rest_logout.pb.h"
#include "rest_post.pb.h"
#include "road.pb.h"
#include "scene.pb.h"
#include "selection.pb.h"
#include "sensor.pb.h"
#include "sensor_noise.pb.h"
#include "server_control.pb.h"
#include "shadows.pb.h"
#include "sim_event.pb.h"
#include "sky.pb.h"
#include "spheregeom.pb.h"
#include "spherical_coordinates.pb.h"
#include "subscribe.pb.h"
#include "surface.pb.h"
#include "tactile.pb.h"
#include "test.pb.h"
#include "time.pb.h"
#include "topic_info.pb.h"
#include "track_visual.pb.h"
#include "twist.pb.h"
#include "undo_redo.pb.h"
#include "user_cmd.pb.h"
#include "user_cmd_stats.pb.h"
#include "vector2d.pb.h"
#include "vector3d.pb.h"
#include "visual.pb.h"
#include "wind.pb.h"
#include "wireless_node.pb.h"
#include "wireless_nodes.pb.h"
#include "world_control.pb.h"
#include "world_reset.pb.h"
#include "world_stats.pb.h"
#include "world_modify.pb.h"
#include "wrench.pb.h"
#include "wrench_stamped.pb.h"

