syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Road 
/// \brief Message for a road


import "vector3d.proto";
import "material.proto";

message Road
{
  required string name          = 1;
  required double width         = 2;
  repeated Vector3d point       = 3;
  optional Material material    = 4;
}
